@@ -12,6 +12,9 @@ from nautobot.extras.jobs import Job, BooleanVar
 from nautobot_circuit_maintenance.models import (
     CircuitImpact,
     CircuitMaintenance,
+    MAX_MAINTENANCE_NAME_LENGTH,
+    MAX_NOTIFICATION_SENDER_LENGTH,
+    MAX_NOTIFICATION_SUBJECT_LENGTH,
     Note,
     ParsedNotification,
     RawNotification,
@@ -20,9 +23,11 @@ from nautobot_circuit_maintenance.models import (
 from .sources import get_notifications, MaintenanceNotification
 
 
+name = "Circuit Maintenance"  # pylint: disable=invalid-name
+
+
 # pylint: disable=broad-except
 PLUGIN_SETTINGS = settings.PLUGINS_CONFIG.get("nautobot_circuit_maintenance", {})
-MAX_INITIAL_DAYS_SINCE = 365
 
 
 def create_circuit_maintenance(
@@ -30,7 +35,7 @@ def create_circuit_maintenance(
 ) -> CircuitMaintenance:
     """Handles the creation of a new circuit maintenance."""
     circuit_maintenance_entry = CircuitMaintenance(
-        name=maintenance_id,
+        name=maintenance_id[:MAX_MAINTENANCE_NAME_LENGTH],
         start_time=datetime.datetime.fromtimestamp(parser_maintenance.start, tz=datetime.timezone.utc),
         end_time=datetime.datetime.fromtimestamp(parser_maintenance.end, tz=datetime.timezone.utc),
         description=parser_maintenance.summary,
@@ -52,7 +57,7 @@ def create_circuit_maintenance(
                 message=f"Circuit ID {circuit.circuit_id} linked to Maintenance {maintenance_id}",
             )
         else:
-            note_entry = Note.objects.create(
+            note_entry, created = Note.objects.get_or_create(
                 maintenance=circuit_maintenance_entry,
                 title=f"Nonexistent circuit ID {circuit.circuit_id}",
                 comment=(
@@ -61,13 +66,14 @@ def create_circuit_maintenance(
                 ),
                 level="WARNING",
             )
-            logger.log_warning(
-                note_entry,
-                message=(
-                    f"Circuit ID {circuit.circuit_id} referenced in {maintenance_id} is not in the Database, adding a "
-                    "note"
-                ),
-            )
+            if created:
+                logger.log_warning(
+                    note_entry,
+                    message=(
+                        f"Circuit ID {circuit.circuit_id} referenced in {maintenance_id} is not in the Database, adding a "
+                        "note"
+                    ),
+                )
     if not CircuitImpact.objects.filter(maintenance=circuit_maintenance_entry):
         logger.log_warning(message=f"Circuit Maintenance {maintenance_id} has none Circuit IDs in the DB.")
 
@@ -161,24 +167,23 @@ def create_or_update_circuit_maintenance(
 
     It returns the CircuitMaintenance entry created or updated.
     """
-    maintenance_id = f"{raw_entry.provider.slug}-{parser_maintenance.maintenance_id}"
+    maintenance_id = f"{raw_entry.provider.slug}-{parser_maintenance.maintenance_id}"[:MAX_MAINTENANCE_NAME_LENGTH]
     try:
         circuit_maintenance_entry = CircuitMaintenance.objects.get(name=maintenance_id)
-        # Using the RawNotification.date as the reference to sort because it's the one that takes into account the
-        # source receving time. The ParsedNotification.date stores the date when the RawNotification was parsed and the
-        # ParsedNotification was created.
+        # Using the RawNotification.stamp as the reference to sort because it's the one that takes into account the
+        # source receving time.
         last_parsed_notification = (
-            circuit_maintenance_entry.parsednotification_set.order_by("raw_notification__date").reverse().last()
+            circuit_maintenance_entry.parsednotification_set.order_by("raw_notification__stamp").reverse().last()
         )
 
         # If the notification is older than the latest one used to update the CircuitMaintenance, we skip updating it
         parser_maintenance_datetime = datetime.datetime.fromtimestamp(
             parser_maintenance.stamp, tz=datetime.timezone.utc
         )
-        if last_parsed_notification and last_parsed_notification.date > parser_maintenance_datetime:
+        if last_parsed_notification and last_parsed_notification.raw_notification.stamp > parser_maintenance_datetime:
             logger.log_debug(
                 f"Not updating CircuitMaintenance {maintenance_id} because the notification is from "
-                f"{parser_maintenance_datetime}, older than the most recent notification from {last_parsed_notification.date}."
+                f"{parser_maintenance_datetime}, older than the most recent notification from {last_parsed_notification.raw_notification.stamp}."
             )
             return circuit_maintenance_entry
 
@@ -223,9 +228,9 @@ def create_raw_notification(logger: Job, notification: MaintenanceNotification,
     """
     try:
         raw_entry = RawNotification.objects.get(
-            subject=notification.subject,
+            subject=notification.subject[:MAX_NOTIFICATION_SUBJECT_LENGTH],
             provider=provider,
-            date=parser.parse(notification.date),
+            stamp=parser.parse(notification.date),
         )
         # If the RawNotification was already created, we ignore it.
         if logger.debug:
@@ -234,12 +239,12 @@ def create_raw_notification(logger: Job, notification: MaintenanceNotification,
     except ObjectDoesNotExist:
         try:
             raw_entry = RawNotification(
-                subject=notification.subject,
+                subject=notification.subject[:MAX_NOTIFICATION_SUBJECT_LENGTH],
                 provider=provider,
                 raw=notification.raw_payload,
-                sender=notification.sender,
+                sender=notification.sender[:MAX_NOTIFICATION_SENDER_LENGTH],
                 source=NotificationSource.objects.filter(name=notification.source).last(),
-                date=parser.parse(notification.date),
+                stamp=parser.parse(notification.date),
             )
             raw_entry.save()
             logger.log_success(raw_entry, message="Raw notification created.")
@@ -304,14 +309,14 @@ def process_raw_notification(logger: Job, notification: MaintenanceNotification)
 
 
 def get_since_reference(logger: Job) -> int:
-    """Get the timestamp from the latest processed RawNotification or a reference from config `initial_days_since`."""
+    """Get the timestamp from the latest processed RawNotification or a reference from config `raw_notification_initial_days_since`."""
     # Latest retrieved notification will limit the scope of notifications to retrieve
     last_raw_notification = RawNotification.objects.last()
     if last_raw_notification:
-        since_reference = last_raw_notification.date.timestamp()
+        since_reference = last_raw_notification.last_updated.timestamp()
     else:
-        since_reference = datetime.datetime.utcnow() - datetime.timedelta(
-            days=PLUGIN_SETTINGS.get("raw_notifications", {}).get("initial_days_since", MAX_INITIAL_DAYS_SINCE)
+        since_reference = datetime.datetime.now(datetime.timezone.utc) - datetime.timedelta(
+            days=PLUGIN_SETTINGS.get("raw_notification_initial_days_since")
         )
         since_reference = int(since_reference.timestamp())
     logger.log_info(message=f"Processing notifications since {since_reference}")
